@@ -7,6 +7,7 @@ use DBD::SQLite qw[];
 use DBD::SQLite::Constants qw[:file_open];
 use Pcore::Util::Scalar qw[weaken is_blessed_ref looks_like_number is_plain_arrayref is_plain_coderef is_blessed_arrayref];
 use Pcore::Util::UUID qw[uuid_str];
+use Pcore::Util::Data qw[to_json];
 
 # NOTE http://habrahabr.ru/post/149635/
 # для вставки данных в цикле надо использовать h->begin_work ... h->commit
@@ -47,20 +48,21 @@ const our $SQLITE_BLOB    => 30;
 
 # postgreSQL types to SQLite
 const our $TYPE_TO_SQLITE => {
-    $SQL_BYTEA   => $SQLITE_BLOB,
     $SQL_BOOL    => $SQLITE_INTEGER,
+    $SQL_BYTEA   => $SQLITE_BLOB,
+    $SQL_CHAR    => $SQLITE_TEXT,
     $SQL_FLOAT4  => $SQLITE_REAL,
     $SQL_FLOAT8  => $SQLITE_REAL,
+    $SQL_JSON    => $SQLITE_BLOB,
     $SQL_INT2    => $SQLITE_INTEGER,
     $SQL_INT4    => $SQLITE_INTEGER,
     $SQL_INT8    => $SQLITE_INTEGER,
     $SQL_MONEY   => $SQLITE_REAL,
     $SQL_NUMERIC => $SQLITE_REAL,
     $SQL_TEXT    => $SQLITE_TEXT,
-    $SQL_VARCHAR => $SQLITE_TEXT,
-    $SQL_CHAR    => $SQLITE_TEXT,
     $SQL_UNKNOWN => $SQLITE_UNKNOWN,
     $SQL_UUID    => $SQLITE_BLOB,
+    $SQL_VARCHAR => $SQLITE_TEXT,
 };
 
 sub BUILD ( $self, $args ) {
@@ -162,7 +164,7 @@ SQL
 }
 
 # QUOTE
-sub _get_sqlite_type ($type = undef) : prototype(;$) {
+sub _get_sqlite_type ($type) : prototype($) {
 
     # use TEXT as default type
     if ( !defined $type || !exists $TYPE_TO_SQLITE->{$type} ) {
@@ -175,10 +177,38 @@ sub _get_sqlite_type ($type = undef) : prototype(;$) {
     return $type;
 }
 
-sub quote ( $self, $var, $type = undef ) {
+sub quote ( $self, $var ) {
     return 'NULL' if !defined $var;
 
-    $type = _get_sqlite_type $type;
+    my $type;
+
+    if ( is_blessed_arrayref $var) {
+        return 'NULL' if !defined $var->[1];
+
+        $type = _get_sqlite_type( $var->[0] );
+
+        if ( $var->[0] == $SQL_BOOL ) {
+            $var = $var->[1] ? 1 : 0;
+        }
+        elsif ( $var->[0] == $SQL_JSON ) {
+            $var = to_json( $var->[1] )->$*;
+        }
+        else {
+            $var = $var->[1];
+        }
+    }
+    else {
+
+        # transparently encode arrays to JSON
+        if ( is_plain_arrayref $var) {
+            $type = $SQLITE_BLOB;
+
+            $var = to_json($var)->$*;
+        }
+        else {
+            $type = $SQLITE_TEXT;
+        }
+    }
 
     # NUMBER
     if ( ( $type == $SQLITE_INTEGER || $type == $SQLITE_REAL ) && looks_like_number $var) {
@@ -284,19 +314,8 @@ sub _exec_sth ( $self, $query, @args ) {
 
     if ( defined $bind ) {
 
-        # make a copy
-        my @bind = $bind->@*;
-
-        # bind types
-        for ( my $i = 0; $i <= $#bind; $i++ ) {
-            if ( is_blessed_arrayref $bind[$i] ) {
-                $sth->bind_param( $i + 1, undef, _get_sqlite_type $bind[$i]->[1] );
-
-                $bind[$i] = $bind[$i]->[1];
-            }
-        }
-
-        $rows = $sth->execute(@bind);
+        # bind and exec
+        $rows = $self->_execute( $sth, $bind, 0 );
     }
     else {
         $rows = $sth->execute;
@@ -322,6 +341,38 @@ sub _die ($self) {
     die qq[DBI: "$DBI::errstr"] . ( defined $self->{query} ? qq[, current query: "$self->{query}->$*"] : q[] );
 }
 
+sub _execute ( $self, $sth, $bind, $bind_pos ) {
+
+    # make a copy
+    my @bind = $bind->@[ $bind_pos .. $bind_pos + $sth->{NUM_OF_PARAMS} - 1 ];
+
+    $bind_pos += $sth->{NUM_OF_PARAMS};
+
+    # bind types
+    for ( my $i = 0; $i <= $#bind; $i++ ) {
+        if ( is_blessed_arrayref $bind[$i] ) {
+            $sth->bind_param( $i + 1, undef, _get_sqlite_type $bind[$i]->[0] );
+
+            if ( $bind[$i]->[0] == $SQL_BOOL ) {
+                $bind[$i] = $bind[$i]->[1] ? 1 : 0;
+            }
+            elsif ( $bind[$i]->[0] == $SQL_JSON ) {
+                $bind[$i] = to_json( $bind[$i]->[1] )->$*;
+            }
+            else {
+                $bind[$i] = $bind[$i]->[1];
+            }
+        }
+        elsif ( is_plain_arrayref $bind[$i] ) {
+            $sth->bind_param( $i + 1, undef, $SQLITE_BLOB );
+
+            $bind[$i] = to_json( $bind[$i] )->$*;
+        }
+    }
+
+    return $sth->execute(@bind);
+}
+
 # PUBLIC DBI METHODS
 sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuiltinHomonyms]
 
@@ -365,22 +416,9 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
             else               { die 'Invalid STH class' }
         }
 
-        # bind params
+        # bind and exec
         if ( defined $bind ) {
-
-            # make a copy
-            my @bind = $bind->@*;
-
-            # bind types
-            for ( my $i = 0; $i <= $#bind; $i++ ) {
-                if ( is_blessed_arrayref $bind[$i] ) {
-                    $sth->bind_param( $i + 1, undef, _get_sqlite_type $bind[$i]->[1] );
-
-                    $bind[$i] = $bind[$i]->[1];
-                }
-            }
-
-            $rows = $sth->execute(@bind);
+            $rows = $self->_execute( $sth, $bind, 0 );
         }
         else {
             $rows = $sth->execute;
@@ -442,21 +480,8 @@ sub do ( $self, $query, @args ) {    ## no critic qw[Subroutines::ProhibitBuilti
                 else               { $self->_die }
             }
 
-            # make a copy
-            my @bind = $bind->@[ $bind_pos .. $bind_pos + $sth->{NUM_OF_PARAMS} - 1 ];
-
-            $bind_pos += $sth->{NUM_OF_PARAMS};
-
-            # bind types
-            for ( my $i = 0; $i <= $#bind; $i++ ) {
-                if ( is_blessed_arrayref $bind[$i] ) {
-                    $sth->bind_param( $i + 1, undef, _get_sqlite_type $bind[$i]->[1] );
-
-                    $bind[$i] = $bind[$i]->[1];
-                }
-            }
-
-            $sth->execute(@bind);
+            # bind and exec
+            $self->_execute( $sth, $bind, $bind_pos );
 
             # execute error
             if ( defined $DBI::err ) {
@@ -682,18 +707,18 @@ sub attach ( $self, $name, $path = undef ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 155                  | Subroutines::ProhibitUnusedPrivateSubroutines - Private subroutine/method '_get_schema_patch_table_query'      |
+## |    3 | 157                  | Subroutines::ProhibitUnusedPrivateSubroutines - Private subroutine/method '_get_schema_patch_table_query'      |
 ## |      |                      | declared but not used                                                                                          |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 326                  | Subroutines::ProhibitExcessComplexity - Subroutine "do" with high complexity score (40)                        |
+## |    3 | 377                  | Subroutines::ProhibitExcessComplexity - Subroutine "do" with high complexity score (36)                        |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 413                  | Subroutines::ProtectPrivateSubs - Private subroutine/method used                                               |
+## |    3 | 451                  | Subroutines::ProtectPrivateSubs - Private subroutine/method used                                               |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 203                  | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
+## |    2 | 233                  | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 291, 375, 451        | ControlStructures::ProhibitCStyleForLoops - C-style "for" loop used                                            |
+## |    2 | 352                  | ControlStructures::ProhibitCStyleForLoops - C-style "for" loop used                                            |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 607                  | ControlStructures::ProhibitPostfixControls - Postfix control "while" used                                      |
+## |    2 | 632                  | ControlStructures::ProhibitPostfixControls - Postfix control "while" used                                      |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
